package boardgame

sealed abstract class List[T]:
  def size: BigInt = {
    this match
      case Nil()  => BigInt(0)
      case h :: t => 1 + t.size
  }.ensuring(_ >= 0)

  def length: BigInt = size

  def ++(that: List[T]): List[T] =
    this match
      case Nil()   => that
      case x :: xs => x :: (xs ++ that)

  def head: T =
    require(this != Nil[T]())
    val h :: _ = this: @unchecked
    h

  def tail: List[T] =
    require(this != Nil[T]())
    val _ :: t = this: @unchecked
    t

  def apply(index: BigInt): T =
    require(0 <= index && index < size)
    if index == 0 then head
    else tail(index - 1)

  def ::(elem: T): List[T] = new ::(elem, this)

  def :+(t: T): List[T] =
    this match
      case Nil()   => t :: this
      case x :: xs => x :: (xs :+ t)

object List:
  def empty[T]: List[T] = Nil()

case class Nil[A]() extends List[A]
case class ::[A](first: A, next: List[A]) extends List[A]
