package boardgame

sealed abstract class Option[T]:
  def get: T =
    require(this.isDefined)
    (this: @unchecked) match
      case Some(x) => x

  def getOrElse(default: => T) =
    this match
      case Some(v) => v
      case None()  => default

  def orElse(or: => Option[T]): Option[T] =
    this match
      case Some(v) => this
      case None()  => or

  def isEmpty =
    this match
      case Some(_) => false
      case None()  => true

  def nonEmpty = !isEmpty

  def isDefined = !isEmpty

  def map[R](f: T => R): Option[R] =
    this match
      case None()  => None[R]()
      case Some(x) => Some(f(x))

  def flatMap[R](f: T => Option[R]): Option[R] =
    this match
      case None()  => None[R]()
      case Some(x) => f(x)

  def filter(p: T => Boolean): Option[T] =
    this match
      case Some(x) if p(x) => Some(x)
      case _               => None[T]()

  def forall(p: T => Boolean) =
    this match
      case Some(x) if !p(x) => false
      case _                => true

  def exists(p: T => Boolean) = !forall(!p(_))

case class Some[T](v: T) extends Option[T]
case class None[T]() extends Option[T]
