package drop

sealed abstract class List[+T]:
  def size: Int = {
    this match
      case Nil => 0
      case h :: t =>
        val tLen = t.size
        if tLen == Int.MaxValue then tLen
        else 1 + tLen
  } ensuring (res => 0 <= res && res <= Int.MaxValue)

  def length: Int = size

  def ++[TT >: T](that: List[TT]): List[TT] =
    this match
      case Nil     => that
      case x :: xs => x :: (xs ++ that)

  def head: T =
    require(this != Nil)
    val h :: _ = this: @unchecked
    h

  def tail: List[T] =
    require(this != Nil)
    val _ :: t = this: @unchecked
    t

  def apply(index: Int): T =
    require(0 <= index && index < size)
    if index == 0 then
      head
    else
      tail(index - 1)

  def ::[TT >: T](elem: TT): List[TT] = new ::(elem, this)

  def :+[TT >: T](t: TT): List[TT] =
    this match
      case Nil     => t :: this
      case x :: xs => x :: (xs :+ t)

  def foldRight[R](z: R)(f: (T, R) => R): R = this match
    case Nil    => z
    case h :: t => f(h, t.foldRight(z)(f))

  def foldLeft[R](z: R)(f: (R, T) => R): R = this match
    case Nil    => z
    case h :: t => t.foldLeft(f(z, h))(f)

  def reverse: List[T] = {
    this match
      case Nil     => this
      case x :: xs => xs.reverse :+ x
  } ensuring (res => res.size == size)

  def contains[TT >: T](v: TT): Boolean = (this match
    case h :: t => h == v || t.contains(v)
    case Nil    => false
  )

object List:
  def empty[T]: List[T] = Nil

case object Nil extends List[Nothing]
final case class ::[+A](first: A, next: List[A]) extends List[A]
