/* Copyright 2022 EPFL, Lausanne */

package drop

/** To run this test suite, start "sbt" then run the "test" command. */
class DropSuite extends munit.FunSuite:

  import Drop.*
  import DropSuiteIO.*

  extension (t: Tests)

    def title: String =
      t match
        case DropWorksOnSimpleCase(ls, n, exp, pts) =>
          s"Drop every $n element in a list of size ${ls.length} ($pts pts)"
        case DropThrowsException(_, n, pts) =>
          s"Exception is thrown when trying to drop $n elements ($pts pts)"

    def msg: String =
      t match
        case DropWorksOnSimpleCase(ls, n, exp, pts) =>
          s"Wrong result when dropping every $n element in a list of size ${ls.length}"
        case DropThrowsException(_, n, _) =>
          s"An Exception should have been thrown when trying to drop $n elements"

    def execute: Unit =
      t match
        case d: DropWorksOnSimpleCase =>
          assertEquals(drop(d.ls, d.n), d.exp, d.msg)
        case e: DropThrowsException =>
          intercept[IllegalArgumentException] {
            drop(e.ls, e.n)
          }

  allTests.foreach(t =>
    test(t.title) { t.execute }
  )

  override val munitTimeout = scala.concurrent.duration.Duration(1, "s")
