import patmat.*
import IntTreeOps.*
import IntTree.*

class IntTreeOpsTest extends munit.FunSuite:
  test("treeMap: identity") {
    (0 until 100).foreach: _ =>
      val TreeConfig(tree, _, _) = generate(maxDepth = 10)
      val tree1 = treeMap(tree, x => x)
      assertEquals(tree, tree1)
  }

  extension (tree: IntTree)
    def sum: Int = treeReduce(tree, _ + _)

  test("treeMap: reset 0") {
    (0 until 100).foreach: _ =>
      val TreeConfig(tree, reducer, mapper) = generate(maxDepth = 10)
      val tree1 = treeMap(tree, x => 0)
      assertEquals(tree1, mapper(x => 0))
      assertEquals(tree1.sum, 0)
  }

  test("treeReduce: sum") {
    (0 until 100).foreach: _ =>
      val TreeConfig(tree, reducer, _) = generate(maxDepth = 10)
      val t = treeReduce(tree, _ + _)
      val answer = reducer(_ + _)
      assertEquals(t, answer)
  }

  test("treeReduce: max") {
    (0 until 100).foreach: _ =>
      val TreeConfig(tree, reducer, _) = generate(maxDepth = 10)
      val t = treeReduce(tree, _ max _)
      val answer = reducer(_ max _)
      assertEquals(t, answer)
  }

  test("treeReduce: prod") {
    (0 until 100).foreach: _ =>
      val TreeConfig(tree, reducer, _) = generate(maxDepth = 10)
      val t = treeReduce(tree, _ * _)
      val answer = reducer(_ * _)
      assertEquals(t, answer)
  }

  test("treeMapReduce: printer") {
    val mapper: Int => String = _.toString
    val reducer: (String, String) => String = (x, y) => f"($x $y)"
    def show(t: IntTree): String = treeMapReduce(t, mapper, reducer)
    assertEquals(show(Leaf(10)), "10")
    assertEquals(show(Branch(Leaf(10), Leaf(20))), "(10 20)")
    assertEquals(show(Branch(Leaf(10), Branch(Leaf(20), Leaf(30)))), "(10 (20 30))")
  }
