package boardgame

object defs:
  case class Tile(
      base: TileBase,
      feature: Option[Feature] = None(),
      resource: Option[Resource] = None(),
      construction: Option[Construction] = None()
  )

  case class Yields(food: BigInt = 0, production: BigInt = 0, coins: BigInt = 0, science: BigInt = 0):
    require(food >= 0 && production >= 0 && coins >= 0 && science >= 0)
    def +(other: Yields): Yields =
      Yields(food + other.food, production + other.production, coins + other.coins, science + other.science)

  // Hexagon tiles, cylinder world (i.e. wraps around x-axis)
  case class WorldMap(tiles: List[Tile], width: BigInt, height: BigInt):
    require(width > 0 && height > 0)
    require(tiles.length == width * height)

  enum TileBase:
    case FlatTerrain(base: BaseTerrain)
    case HillTerrain(base: BaseTerrain)
    case Mountain
    case Lake
    case Coast
    case Ocean

  enum BaseTerrain:
    case Plains
    case Grassland
    case Desert
    case Tundra
    case Snow

  enum Feature:
    case Woods
    case RainForest
    case Marsh
    case Floodplains
    case Oasis
    case GeothermalFissure
    case Reef

  enum Resource:
    case Crabs
    case Fish
    case Rice
    case Stone
    case Wheat
    case Aluminum
    case Coal
    case Iron

  enum Construction:
    case City(id: BigInt)
    case District(kind: DistrictKind)
    case Exploitation(kind: ResourceImprovement)

    def isCityOrDistrict: Boolean = this match
      case City(_)         => true
      case District(_)     => true
      case Exploitation(_) => false

  enum DistrictKind:
    case Campus
    case IndustrialZone
    case CommercialHub

  enum ResourceImprovement:
    case Farm
    case FishingBoats
    case LumberMill
    case Mine
    case Quarry
