/* Copyright 2024 EPFL, Lausanne */

package boardgame

object BoardGameSuiteIO:

  import defs.*
  import BoardGame.*

  sealed abstract class Tests
  case class WarmupTest(x: Int, y: Int, exp: Tile, points: Int) extends Tests
  case class Task1Test(tile: Tile, exp: Yields, points: Int) extends Tests
  case class Task2Test(x: Int, y: Int, exp: BigInt, points: Int) extends Tests
  case class Task3aTest(x: Int, y: Int, radius: BigInt, exp: List[Tile], points: Int) extends Tests
  case class Task3bTest(x: Int, y: Int, radius: BigInt, exp: List[Tile], points: Int) extends Tests
  case class Task3Test(wm: WorldMap, x: Int, y: Int, exp: Boolean, points: Int) extends Tests

  object warmup:
    val G = Tile(TileBase.FlatTerrain(BaseTerrain.Grassland))
    val X = Tile(TileBase.HillTerrain(BaseTerrain.Desert))
    val Y = Tile(TileBase.Mountain)
    val Z = Tile(TileBase.Lake)
    val W = Tile(TileBase.Ocean)
    val tiles =
      (
        G :: Z :: G :: G :: W ::
          G :: G :: G :: Y :: G ::
          G :: X :: G :: G :: G ::
          G :: G :: G :: G :: G :: Nil()
      )
    val wm = WorldMap(tiles, 5, 4)
    val allTests: Seq[Tests] = Seq(
      WarmupTest(1, 1, X, 10),
      WarmupTest(6, 1, X, 10),
      WarmupTest(-4, 1, X, 10),
      WarmupTest(3, 2, Y, 10),
      WarmupTest(13, 2, Y, 10),
      WarmupTest(1, 3, Z, 10),
      WarmupTest(4, 3, W, 10),
      WarmupTest(9, 3, W, 10)
    )

  object task1:
    val M1 = Tile(TileBase.Mountain)
    val L1 = Tile(TileBase.Lake)
    val L2 = Tile(
      TileBase.Lake,
      Some(Feature.Reef),
      Some(Resource.Crabs),
      Some(Construction.Exploitation(ResourceImprovement.FishingBoats))
    )
    val L3 = Tile(TileBase.Lake, None(), Some(Resource.Fish), None())
    val G1 = Tile(TileBase.FlatTerrain(BaseTerrain.Grassland))
    val G2 = Tile(
      TileBase.FlatTerrain(BaseTerrain.Grassland),
      Some(Feature.Floodplains),
      Some(Resource.Wheat),
      Some(Construction.Exploitation(ResourceImprovement.Farm))
    )
    val P1 = Tile(
      TileBase.HillTerrain(BaseTerrain.Plains),
      Some(Feature.Woods),
      None(),
      Some(Construction.Exploitation(ResourceImprovement.LumberMill))
    )
    val allTests: Seq[Tests] = Seq(
      Task1Test(M1, Yields(), 10),
      Task1Test(L1, Yields(food = 1, coins = 1), 10),
      Task1Test(L2, Yields(food = 3, coins = 3, science = 1), 10),
      Task1Test(L3, Yields(food = 2, coins = 1), 10),
      Task1Test(G1, Yields(food = 2), 10),
      Task1Test(G2, Yields(food = 5, production = 1), 10),
      Task1Test(P1, Yields(food = 1, production = 5), 10)
    )

  object task2:
    val G = Tile(TileBase.FlatTerrain(BaseTerrain.Grassland))
    val M = Tile(TileBase.Mountain)
    val C = Tile(TileBase.FlatTerrain(BaseTerrain.Grassland), construction = Some(Construction.City(42)))
    val K =
      Tile(TileBase.FlatTerrain(BaseTerrain.Grassland), construction = Some(Construction.District(DistrictKind.Campus)))
    val I = Tile(
      TileBase.FlatTerrain(BaseTerrain.Grassland),
      construction = Some(Construction.District(DistrictKind.IndustrialZone))
    )
    val L = Tile(
      TileBase.FlatTerrain(BaseTerrain.Grassland),
      construction = Some(Construction.Exploitation(ResourceImprovement.LumberMill))
    )
    val tiles =
      (
        G :: M :: K :: M :: G ::
          G :: G :: C :: M :: G ::
          G :: M :: I :: I :: G ::
          G :: G :: L :: L :: G :: Nil()
      )
    val wm = WorldMap(tiles, 5, 4)
    val allTests: Seq[Tests] = Seq(
      Task2Test(2, 3, 3, 10),
      Task2Test(2, 1, 2, 10),
      Task2Test(3, 1, 1, 10),
      Task2Test(3, 2, 0, 10),
      Task2Test(2, 2, 0, 10)
    )
  object task3ab:
    // We use cities since these have identifiers to ensure tiles are distinct
    def C(id: BigInt): Tile =
      Tile(TileBase.FlatTerrain(BaseTerrain.Grassland), construction = Some(Construction.City(id)))
    // The center of the ring
    val XXXXX = Tile(TileBase.FlatTerrain(BaseTerrain.Grassland))
    // Note: prefixing < 10 with a zero for alignment (and no, in Scala, it is not treated as octal)
    val tiles = (
      C(1) :: C(2) :: C(3) :: C(4) :: C(5) :: C(6) :: C(7) ::
        C(8) :: C(9) :: C(10) :: C(11) :: C(12) :: C(13) :: C(14) ::
        C(15) :: XXXXX :: C(16) :: C(17) :: C(18) :: C(19) :: C(20) ::
        C(21) :: C(22) :: C(23) :: C(24) :: C(25) :: C(26) :: C(27) :: Nil()
    )
    val wm = WorldMap(tiles, 7, 4)
    val r1 = C(9) :: C(16) :: C(23) :: C(22) :: C(15) :: C(8) :: Nil()
    val r2 = C(2) :: C(10) :: C(17) :: C(24) :: C(21) :: C(20) :: C(14) :: C(7) :: C(1) :: Nil()
    val r3 = C(3) :: C(11) :: C(18) :: C(25) :: C(27) :: C(19) :: C(13) :: C(6) :: Nil()
    val allTests: Seq[Tests] = Seq(
      Task3aTest(1, 1, 0, XXXXX :: Nil(), 10),
      Task3aTest(1, 1, 1, r1, 10),
      Task3aTest(1, 1, 2, r2, 10),
      Task3aTest(1, 1, 3, r3, 10),
      Task3bTest(1, 1, 0, XXXXX :: Nil(), 10),
      Task3bTest(1, 1, 1, XXXXX :: r1, 10),
      Task3bTest(1, 1, 2, XXXXX :: r1 ++ r2, 10),
      Task3bTest(1, 1, 3, XXXXX :: r1 ++ r2 ++ r3, 10)
    )

  object task3:
    val test1: Tests =
      val G = Tile(TileBase.FlatTerrain(BaseTerrain.Grassland))
      val C = Tile(TileBase.FlatTerrain(BaseTerrain.Grassland), construction = Some(Construction.City(1)))
      val X = G // Spot of interest, nok
      val wm =
        (
          G :: G :: G :: G :: G ::
            G :: G :: G :: G :: C ::
            G :: X :: G :: G :: G ::
            G :: G :: G :: G :: G :: Nil()
        )
      Task3Test(WorldMap(wm, 5, 4), 1, 1, false, 10)

    val test2: Tests =
      val G = Tile(TileBase.FlatTerrain(BaseTerrain.Grassland))
      val C = Tile(TileBase.FlatTerrain(BaseTerrain.Grassland), construction = Some(Construction.City(1)))
      val X = G // Spot of interest, ok
      val wm =
        (
          G :: G :: C :: G :: G ::
            G :: G :: G :: G :: G ::
            G :: X :: G :: G :: G ::
            G :: G :: G :: G :: G :: Nil()
        )
      Task3Test(WorldMap(wm, 5, 4), 1, 1, true, 10)

    val test3: Tests =
      val G = Tile(TileBase.FlatTerrain(BaseTerrain.Grassland))
      val C = Tile(TileBase.FlatTerrain(BaseTerrain.Grassland), construction = Some(Construction.City(1)))
      val X = G // Spot of interest, nok
      val wm =
        (
          G :: G :: C :: G :: G ::
            G :: G :: G :: G :: G ::
            G :: X :: G :: G :: C ::
            G :: G :: G :: G :: G :: Nil()
        )
      Task3Test(WorldMap(wm, 5, 4), 1, 1, false, 10)

    val test4: Tests =
      val G = Tile(TileBase.FlatTerrain(BaseTerrain.Grassland))
      val C = Tile(TileBase.FlatTerrain(BaseTerrain.Grassland), construction = Some(Construction.City(1)))
      val X = G // Spot of interest, ok
      val wm =
        (
          G :: G :: G :: C :: G ::
            G :: G :: G :: G :: G ::
            G :: X :: G :: G :: G ::
            G :: G :: G :: G :: G :: Nil()
        )
      Task3Test(WorldMap(wm, 5, 4), 1, 1, true, 10)

    val allTests: Seq[Tests] = Seq(test1, test2, test3, test4)

  val allTests = warmup.allTests ++ task1.allTests ++ task2.allTests ++ task3ab.allTests ++ task3.allTests
